package com.alanyates.cellularAutomata;

import java.awt.image.*;
import java.applet.*;
import java.awt.*;
import java.util.Random;

public class CellularAutomata extends Applet implements Runnable {

	private int buffer[];
	private Image image;
	private MemoryImageSource imageSource;

	private double scaleX = 10;
	private double scaleY = 10;
	private double scaleZ = 10;

	private Thread thread;
	private boolean running;

	public void init() {
		this.buffer = new int[getSize().width*getSize().height];
		this.imageSource = new MemoryImageSource(getSize().width, getSize().height, new DirectColorModel(32, 0xff0000, 0xff00, 0xff), this.buffer, 0, getSize().width);
		this.imageSource.setAnimated(true);
		this.image = Toolkit.getDefaultToolkit().createImage(this.imageSource);		
	}

	public void start() {
		if(this.thread == null) {
			this.running = true;
			this.thread = new Thread(this);
			this.thread.start();
		}
	}

	public void stop() {
		if(this.thread != null && this.thread.isAlive()) {
			this.running = false;
		}
	}
	
	public void paint() {
		this.imageSource.newPixels();
		getGraphics().drawImage(this.image, 0, 0, getSize().width, getSize().height, null);
	}

	public void run() {
		System sys = null;
		int delay = 1000;
		int resetAtGeneration = 0;
		try {
			sys = new System(this);
 		} catch(Exception ex) {
			ex.printStackTrace();
		}
		try { delay = Integer.parseInt(sys.getParameter("frameDelay")); } catch(Exception ignored) {}
		try { resetAtGeneration = Integer.parseInt(sys.getParameter("resetAtGeneration")); } catch(Exception ignored) {}
		java.lang.System.out.println("delay = " + delay);
 		
		while(sys != null && this.running) {
			sys.step();
			sys.draw();
			this.paint();
			try { if(delay > 0) Thread.sleep(delay); } catch(Exception ignored) {}
			try { if(resetAtGeneration > 0 && sys.generation > resetAtGeneration) sys.initCells(); } catch(Exception ignored) {}
		}
	}

	public void setPixel(int x, int y, int c) {
		if(x < 0 || x > getSize().width-1) return;
		if(y < 0 || y > getSize().height-1) return;
		this.buffer[x + (y*getSize().width)] = c;
	}
}
