package com.alanyates.cellularAutomata;

public class System {
	private CellularAutomata container;

	private int rows;
	private int cols;
	public int generation;

	private boolean closed;
	
	private Cell[] cells;
	
	public System(CellularAutomata container) throws ClassNotFoundException, InstantiationException, IllegalAccessException {
		this.container = container;
		initCells();
	}
	
	public void initCells() throws ClassNotFoundException, InstantiationException, IllegalAccessException {
		this.generation = 0;
		this.cols = Integer.parseInt(getParameter("cols"));
		this.rows = Integer.parseInt(getParameter("rows"));

		this.closed = !"false".equals(getParameter("closed"));

		CellFactory cellMaker = new CellFactory(this);
		this.cells = new Cell[this.rows*this.cols];
		for(int y = 0; y < this.rows; y++) {
			for(int x = 0; x < this.cols; x++) {
				this.cells[x + (y*this.cols)] = cellMaker.getNewInstance(x, y);
			}
		}
	}
	
	public Cell cellAt(int x, int y) {
		if(this.closed) {
			x %= this.cols;
			y %= this.rows;
			if(x < 0) x += this.cols;
			if(y < 0) y += this.rows;
		} else {
			if(x < 0 || y < 0 || x > this.cols - 1 || y > this.rows - 1) return null;
		}

		return this.cells[x + (y*this.cols)];
	}

	public String getParameter(String name) {
		return this.container.getParameter(name);
	}

	public void step() {
		for(int i = 0; i < this.cols * this.rows; i++) cells[i].calculateNextState(this);
		for(int i = 0; i < this.cols * this.rows; i++) cells[i].transitionToNextState(this);
		this.generation++;
	}
	
	public void draw() {
		int cellWidth = this.container.getSize().width / this.cols;
		int cellHeight = this.container.getSize().height / this.rows;
		
		for(int y = 0; y < this.rows; y++) {
			for(int x = 0; x < this.cols; x++) {
				rect(x * cellWidth, y * cellHeight, cellWidth, cellHeight, cellAt(x, y).getColour());
			}
		}
	}
	
	private void rect(int x, int y, int xw, int yw, int c) {
		for(int i = 0; i < xw; i++) {
			for(int j = 0; j < yw; j++) {
				this.container.setPixel(x+i, y+j, c);
			}
		}
	}
}
